@@ -88,11 +88,7 @@ struct GSWindowPrivate
 	GtkWidget *info_bar;
 	GtkWidget *info_content;
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	cairo_surface_t *background_surface;
-#else
-	GdkPixmap *background_pixmap;
-#endif
 
 	guint      popup_dialog_idle_id;
 
@@ -266,18 +262,7 @@ widget_clear_all_children (GtkWidget *widget)
 #endif
 }
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-static void
-gs_window_reset_background_surface (GSWindow *window)
-{
-	cairo_pattern_t *pattern;
-	pattern = cairo_pattern_create_for_surface (window->priv->background_surface);
-	gdk_window_set_background_pattern (gtk_widget_get_window (GTK_WIDGET (window)),
-									   pattern);
-	cairo_pattern_destroy (pattern);
-	gtk_widget_queue_draw (GTK_WIDGET (window));
-}
-#else
+#if !GTK_CHECK_VERSION (3, 0, 0)
 static void
 force_no_pixmap_background (GtkWidget *widget)
 {
@@ -307,177 +292,23 @@ force_no_pixmap_background (GtkWidget *widget)
 #endif
 
 void
-#if GTK_CHECK_VERSION (3, 0, 0)
 gs_window_set_background_surface (GSWindow        *window,
                                   cairo_surface_t *surface)
-#else
-gs_window_set_background_pixmap (GSWindow  *window,
-                                 GdkPixmap *pixmap)
-#endif
 {
 	g_return_if_fail (GS_IS_WINDOW (window));
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	if (window->priv->background_surface != NULL)
 	{
 		cairo_surface_destroy (window->priv->background_surface);
 	}
+	window->priv->background_surface = NULL;
 
 	if (surface != NULL)
 	{
 		window->priv->background_surface = cairo_surface_reference (surface);
-		gs_window_reset_background_surface (window);
-	}
-#else
-	if (window->priv->background_pixmap != NULL)
-	{
-		g_object_unref (window->priv->background_pixmap);
 	}
 
-	if (pixmap != NULL)
-	{
-		window->priv->background_pixmap = g_object_ref (pixmap);
-		gdk_window_set_back_pixmap (gtk_widget_get_window (GTK_WIDGET (window)),
-		                            pixmap,
-		                            FALSE);
-	}
-#endif
-}
-
-static void
-#if GTK_CHECK_VERSION (3, 0, 0)
-gs_window_clear_to_background_surface (GSWindow *window)
-#else
-gs_window_clear_to_background_pixmap (GSWindow *window)
-#endif
-{
-#if !GTK_CHECK_VERSION (3, 0, 0)
-	GtkStateType state;
-	GtkStyle    *style;
-#endif
-
-	g_return_if_fail (GS_IS_WINDOW (window));
-
-	if (! gtk_widget_get_visible (GTK_WIDGET (window)))
-	{
-		return;
-	}
-
-#if GTK_CHECK_VERSION (3, 0, 0)
-	if (window->priv->background_surface == NULL)
-#else
-	if (window->priv->background_pixmap == NULL)
-#endif
-	{
-		/* don't allow null pixmaps */
-		return;
-	}
-
-	gs_debug ("Clearing window to background pixmap");
-
-#if GTK_CHECK_VERSION (3, 0, 0)
-	gs_window_reset_background_surface (window);
-#else
-	style = gtk_style_copy (gtk_widget_get_style (GTK_WIDGET (window)));
-
-	state = (GtkStateType) 0;
-	while (state < (GtkStateType) G_N_ELEMENTS (gtk_widget_get_style (GTK_WIDGET (window))->bg_pixmap))
-	{
-
-		if (style->bg_pixmap[state] != NULL)
-		{
-			g_object_unref (style->bg_pixmap[state]);
-		}
-
-		style->bg_pixmap[state] = g_object_ref (window->priv->background_pixmap);
-		state++;
-	}
-
-	gtk_widget_set_style (GTK_WIDGET (window), style);
-	g_object_unref (style);
-
-	if (window->priv->background_pixmap != NULL)
-	{
-		gdk_window_set_back_pixmap (gtk_widget_get_window (GTK_WIDGET (window)),
-		                            window->priv->background_pixmap,
-		                            FALSE);
-	}
-
-	gdk_window_clear (gtk_widget_get_window (GTK_WIDGET (window)));
-
-	gdk_flush ();
-#endif
-}
-
-static void
-clear_widget (GtkWidget *widget)
-{
-#if GTK_CHECK_VERSION (3, 0, 0)
-	GdkRGBA       rgba = { 0.0, 0.0, 0.0, 1.0 };
-	GtkStateFlags state;
-#else
-	GdkColormap *colormap;
-	GdkColor     color = { 0, 0x0000, 0x0000, 0x0000 };
-	GtkStateType state;
-	GtkStyle    *style;
-#endif
-
-#if GTK_CHECK_VERSION (3, 0, 0)
-	if (!gtk_widget_get_realized (widget))
-	{
-		return;
-	}
-
-	gs_debug ("Clearing widget");
-
-	state = gtk_widget_get_state_flags (widget);
-	gtk_widget_override_background_color (widget, state, &rgba);
-	gdk_window_set_background_rgba (gtk_widget_get_window (widget), &rgba);
-	gtk_widget_queue_draw (GTK_WIDGET (widget));
-#else
-	if (!gtk_widget_get_visible (widget))
-	{
-		return;
-	}
-
-	gs_debug ("Clearing widget");
-
-	state = (GtkStateType) 0;
-	while (state < (GtkStateType) G_N_ELEMENTS (gtk_widget_get_style (widget)->bg))
-	{
-		gtk_widget_modify_bg (widget, state, &color);
-		state++;
-	}
-
-	style = gtk_style_copy (gtk_widget_get_style (widget));
-
-	state = (GtkStateType) 0;
-	while (state < (GtkStateType) G_N_ELEMENTS (gtk_widget_get_style (widget)->bg_pixmap))
-	{
-
-		if (style->bg_pixmap[state] != NULL)
-		{
-			g_object_unref (style->bg_pixmap[state]);
-		}
-
-		style->bg_pixmap[state] = NULL;
-		state++;
-	}
-
-	colormap = gdk_drawable_get_colormap (gtk_widget_get_window (widget));
-	gdk_colormap_alloc_color (colormap, &color, FALSE, TRUE);
-	gdk_window_set_background (gtk_widget_get_window (widget), &color);
-
-	gtk_widget_set_style (widget, style);
-	g_object_unref (style);
-
-	gdk_window_clear (gtk_widget_get_window (widget));
-#endif
-
-	/* If a screensaver theme adds child windows we need to clear them too */
-	widget_clear_all_children (widget);
-
-	gdk_flush ();
+	gtk_widget_queue_draw (GTK_WIDGET (window));
 }
 
 void
@@ -485,8 +316,23 @@ gs_window_clear (GSWindow *window)
 {
 	g_return_if_fail (GS_IS_WINDOW (window));
 
-	clear_widget (GTK_WIDGET (window));
-	clear_widget (window->priv->drawing_area);
+	gs_debug ("Clearing widgets");
+
+	if (gtk_widget_get_realized (GTK_WIDGET (window)))
+	{
+		gtk_widget_queue_draw (GTK_WIDGET (window));
+		/* If a screensaver theme adds child windows
+		   we need to clear them too */
+		widget_clear_all_children (GTK_WIDGET (window));
+	}
+
+	if (gtk_widget_get_realized (window->priv->drawing_area))
+	{
+		gtk_widget_queue_draw (window->priv->drawing_area);
+		widget_clear_all_children (window->priv->drawing_area);
+	}
+
+	gdk_flush ();
 }
 
 #if GTK_CHECK_VERSION (3, 0, 0)
@@ -1071,6 +917,39 @@ window_select_shape_events (GSWindow *window)
 #endif
 }
 
+static gboolean
+#if GTK_CHECK_VERSION (3, 0, 0)
+gs_window_real_draw (GtkWidget *widget,
+                     cairo_t   *cr)
+#else
+gs_window_real_expose_event (GtkWidget      *widget,
+                             GdkEventExpose *event)
+#endif
+{
+	GSWindow *window = GS_WINDOW (widget);
+	cairo_surface_t *bg_surface = window->priv->background_surface;
+#if !GTK_CHECK_VERSION (3, 0, 0)
+	GdkWindow *gdk_window = gtk_widget_get_window (widget);
+	cairo_t *cr = gdk_cairo_create (gdk_window);
+#endif
+
+	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+	if (bg_surface != NULL)
+	{
+		cairo_set_source_surface (cr, bg_surface, 0, 0);
+	}
+	else
+	{
+		cairo_set_source_rgb (cr, 0, 0, 0);
+	}
+	cairo_paint (cr);
+
+#if !GTK_CHECK_VERSION (3, 0, 0)
+	cairo_destroy (cr);
+#endif
+	return FALSE;
+}
+
 static void
 gs_window_real_show (GtkWidget *widget)
 {
@@ -1779,7 +1658,7 @@ shake_dialog (GSWindow *window)
 
 	window->priv->dialog_shake_in_progress = TRUE;
 
-	for (i = 0; i < 9; i++)
+	for (i = 0; i < 8; i++)
 	{
 		if (i % 2 == 0)
 		{
@@ -2027,11 +1906,7 @@ popup_dialog (GSWindow *window)
 
 	gtk_widget_hide (window->priv->drawing_area);
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-	gs_window_clear_to_background_surface (window);
-#else
-	gs_window_clear_to_background_pixmap (window);
-#endif
+	gtk_widget_queue_draw (GTK_WIDGET (window));
 	set_invisible_cursor (gtk_widget_get_window (GTK_WIDGET (window)), FALSE);
 
 	window->priv->dialog_quit_requested = FALSE;
@@ -2633,6 +2508,11 @@ gs_window_class_init (GSWindowClass *klass)
 
 	widget_class->show                = gs_window_real_show;
 	widget_class->hide                = gs_window_real_hide;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	widget_class->draw                = gs_window_real_draw;
+#else
+	widget_class->expose_event        = gs_window_real_expose_event;
+#endif
 	widget_class->realize             = gs_window_real_realize;
 	widget_class->unrealize           = gs_window_real_unrealize;
 	widget_class->key_press_event     = gs_window_real_key_press_event;
@@ -2757,16 +2637,29 @@ create_info_bar (GSWindow *window)
 	gtk_box_pack_end (GTK_BOX (window->priv->vbox), window->priv->info_bar, FALSE, FALSE, 0);
 }
 
+static gboolean
 #if GTK_CHECK_VERSION (3, 0, 0)
-static void
-on_drawing_area_realized (GtkWidget *drawing_area)
-{
-	GdkRGBA black = { 0.0, 0.0, 0.0, 1.0 };
-
-	gdk_window_set_background_rgba (gtk_widget_get_window (drawing_area),
-	                                &black);
-}
+on_drawing_area_draw (GtkWidget *widget,
+                      cairo_t   *cr)
+#else
+on_drawing_area_expose_event (GtkWidget      *widget,
+                              GdkEventExpose *event)
 #endif
+{
+#if !GTK_CHECK_VERSION (3, 0, 0)
+	GdkWindow *window = gtk_widget_get_window (widget);
+	cairo_t *cr = gdk_cairo_create (window);
+
+#endif
+	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+	cairo_set_source_rgb (cr, 0, 0, 0);
+	cairo_paint (cr);
+
+#if !GTK_CHECK_VERSION (3, 0, 0)
+	cairo_destroy (cr);
+#endif
+	return FALSE;
+}
 
 static void
 gs_window_init (GSWindow *window)
@@ -2812,21 +2705,22 @@ gs_window_init (GSWindow *window)
 
 	window->priv->drawing_area = gtk_drawing_area_new ();
 	gtk_widget_show (window->priv->drawing_area);
-#if GTK_CHECK_VERSION (3, 0, 0)
 	gtk_widget_set_app_paintable (window->priv->drawing_area, TRUE);
-#endif
-	gtk_box_pack_start (GTK_BOX (window->priv->vbox), window->priv->drawing_area, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (window->priv->vbox),
+	                    window->priv->drawing_area, TRUE, TRUE, 0);
 #if GTK_CHECK_VERSION (3, 0, 0)
 	g_signal_connect (window->priv->drawing_area,
-	                  "realize",
-	                  G_CALLBACK (on_drawing_area_realized),
+	                  "draw",
+	                  G_CALLBACK (on_drawing_area_draw),
+	                  NULL);
+#else
+	g_signal_connect (window->priv->drawing_area,
+	                  "expose-event",
+	                  G_CALLBACK (on_drawing_area_expose_event),
 	                  NULL);
-#endif
-	create_info_bar (window);
-
-#if !GTK_CHECK_VERSION (3, 0, 0)
 	force_no_pixmap_background (window->priv->drawing_area);
 #endif
+	create_info_bar (window);
 }
 
 static void
@@ -2879,17 +2773,10 @@ gs_window_finalize (GObject *object)
 
 	gs_window_dialog_finish (window);
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	if (window->priv->background_surface)
 	{
 		cairo_surface_destroy (window->priv->background_surface);
 	}
-#else
-	if (window->priv->background_pixmap)
-	{
-		g_object_unref (window->priv->background_pixmap);
-	}
-#endif
 
 	G_OBJECT_CLASS (gs_window_parent_class)->finalize (object);
 }
